package cs636.pizza.service;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;

import cs636.pizza.config.PizzaSystemConfig;

public class AdminServiceTest {
	private AdminService adminService;
	private StudentService studentService;

	@BeforeEach
	public void setUp() throws Exception {
		// Since we're setting up multiple objects here, this is not a real "unit test"
		// but rather an "integration test"
		PizzaSystemConfig.configureServices(null, null, null); // use defaults (H2)
		adminService = PizzaSystemConfig.getAdminService();
		studentService = PizzaSystemConfig.getStudentService();
		adminService.initializeDb(); // bring system to known state
	}

	@AfterEach
	public void tearDown() throws Exception {
		PizzaSystemConfig.shutdownServices();
	}

	@Test
	public void testAddTopping() throws ServiceException {
		adminService.addTopping("mushrooms");
		Assertions.assertTrue(studentService.getToppingNames().size() == 1, () -> "first topping not created");
	}

	@Test
	public void testAddPizzaSize() throws ServiceException {
		adminService.addSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 1, () -> "first size not created");
	}

	@Test
	public void testAddRemovePizzaSize() throws ServiceException {
		adminService.addSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 1, () -> "first size not created");
		adminService.removeSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 0, () -> "first size created and removed");
	}

	// duplicate will cause unique constraint failure
	@Test
	public void testAddDupPizzaSize() throws ServiceException {
		Assertions.assertThrows(ServiceException.class, () -> {
			adminService.addSize("small");
			adminService.addSize("small");
		});
	}

	// you can't delete a nonexistent size quietly
	@Test
	public void testRemoveNonExistentPizzaSize() throws ServiceException {
		Assertions.assertThrows(ServiceException.class, () -> {
			adminService.removeSize("small");
		});
	}

	@Test
	public void testGetPizzaSizes() throws ServiceException {
		Assertions.assertTrue(studentService.getSizeNames().size() == 0, () -> "sizes exist at init (should be none)");
		adminService.addSize("small");
		Assertions.assertTrue(studentService.getSizeNames().size() == 1, () -> "first added size not there");
		Assertions.assertTrue(studentService.getSizeNames().iterator().next().equals("small"),
				() -> "first added size not right");
	}

	@Test
	public void testAddTopping1() throws ServiceException {
		Assertions.assertTrue(studentService.getToppingNames().size() == 0,
				() -> "toppings exist at init (should be none)");
		adminService.addTopping("xxx");
		Assertions.assertTrue(studentService.getToppingNames().size() == 1, () -> "first added topping not there");
		Assertions.assertTrue(studentService.getToppingNames().iterator().next().equals("xxx"),
				() -> "first added topping not right");

	}

}
