package cs636.pizza.service;

import java.util.Set;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;

import cs636.pizza.config.PizzaSystemConfig;

public class DeleteToppingTest {
	private AdminService adminService;
	private StudentService studentService;
	
	@BeforeEach
	public void setUp() throws Exception {
		// Since we're setting up multiple objects here, this is not a real "unit test"
		// but rather an "integration test"
		PizzaSystemConfig.configureServices(null, null, null); // use defaults
		adminService = PizzaSystemConfig.getAdminService();
		studentService = PizzaSystemConfig.getStudentService();
		adminService.initializeDb(); // bring system to known state
		adminService.addTopping("xxx"); // set up a topping and size
		adminService.addSize("yyy");
	}

	@AfterEach
	public void tearDown() throws Exception {
		// This executes even after an exception
		// do full shutdown to make sure that
		// even if a transaction is still going
		// it gets cleaned up
		PizzaSystemConfig.shutdownServices();	
	}
	
	// one user selects a topping, another deletes it
	// then first user orders with it
	// Note: with fancier code, we could check that the
	// exception message has "Topping" in it, as expected
	@Test
	public void testDropToppingMakeOrder() throws ServiceException  {
		Assertions.assertThrows(ServiceException.class, () -> {
		// user 1 action
		Set<String> tops = studentService.getToppingNames();
		// user 2 action
		adminService.removeTopping("xxx");  // xxx was added in setUp
		// user1 actions
		String size = studentService.getSizeNames().iterator().next();
		studentService.makeOrder(1, size, tops);
		});
	}

}
